(**************************************************************************)
(*  The CDuce interpreter                                                 *)
(*  Alain.Frisch@ens.fr                                                   *)
(*  Copyright 2003 (see LICENSE for details)                              *)
(**************************************************************************)

module type T = 
sig
  type t  
  type value
    
  val clear: unit -> unit  
  val mk: value -> t
  val dummy_min: t
  val dummy_max: t

  val value: t -> value

  val compare: t -> t -> int
  val hash: t -> int
  val equal: t -> t -> bool
end

module Make(H : Hashtbl.HashedType) =
struct
  type t = int
  type value = H.t

  module Tbl = Hashtbl.Make(H)

  let cache    = State.ref "Pool.cache" (Tbl.create 63)
  let values   = State.ref "Pool.values" (Array.create 63 None)
  let counter  = State.ref "Pool.counter" 0

  let clear () =
    Tbl.clear !cache;
    values := Array.create 63 None;
    counter := 0

  let mk x =
    try Tbl.find !cache x
    with Not_found ->
      let n = !counter in
      incr counter;
      Tbl.add !cache x n;
      if (n = Array.length !values) then
	( 
	  let new_values = Array.create (2 * Array.length !values) None in
	  Array.blit !values 0 new_values 0 n;
	  values := new_values
	);
      !values.(n) <- Some x;
      n

  let dummy_min = -1
  let dummy_max = max_int

  let value n = match !values.(n) with Some x -> x | None -> assert false

  let compare (n1 : int) (n2 : int) = 
    if n1 < n2 then -1 else if n1 = n2 then 0 else 1
  let hash n = n
  let equal (n1 : int) (n2 : int) = n1 = n2
end

