open Ident
open Encodings

type t =
(* Canonical representation *)
  | Pair of t * t
  | Xml of t * t * t
  | Record of t label_map
  | Atom of Atoms.v
  | Integer of Intervals.v
  | Char of Chars.v
  | Abstraction of (Types.descr * Types.descr) list * (t -> t)

(* Derived forms *)
  | String_latin1 of int * int * string * t
  | String_utf8 of U.uindex * U.uindex * U.t * t

(* Special value for absent record fields, and failed pattern matching *)
  | Absent

(* Only in evaluation environment *)
  | Delayed of t ref

exception CDuceExn of t

val print: Format.formatter -> t -> unit

val normalize: t -> t
  (* Transform a derived form to its canonical equivalent *)

val const : Types.const -> t
val string_latin1 : string -> t
val string_utf8 : U.t -> t
val nil : t
val vtrue : t
val vfalse : t
val vbool : bool -> t

  (** @return a Record value from an associative list of fields *)
val vrecord : (string * t) list -> t

val sequence : t list -> t
  (* iterator on the content of an Xml value. First callback is invoked on Utf8
  character children; second callback is invoked on other children values *)
val iter_xml : (U.t -> unit) -> (t -> unit) -> t -> unit
val concat : t -> t -> t
val flatten : t -> t

val get_string_latin1 : t -> string
val get_string_utf8 : t -> U.t * t
val is_str : t -> bool

val get_int : t -> int

  (** @return an associative list of fields from a Record value *)
val get_fields : t -> (string * t) list

val compare : t -> t -> int


val set_cdr : t -> t -> unit
val append_cdr : t -> t -> t
