(**************************************************************************)
(*  The CDuce interpreter                                                 *)
(*  Alain.Frisch@ens.fr                                                   *)
(*  Copyright 2003 (see LICENSE for details)                              *)
(**************************************************************************)

(* Implementation of boolean combination in disjunctive normal form (dnf).
   A dnf is a disjunction of lines, each line consisting of a 
   conjunction of atoms and negations of atoms.

   A line is coded as a pair (p,n) where p collects atoms and n
   the negated atoms. p and n are disjoint and sorted lists without duplicates
   (w.r.t Pervasives.compare).

   A dnf is coded as a sorted list of lines without duplicated
   (w.r.t Pervasives.compare).
*)

module type S =
sig
  type 'a elem
  type 'a t 

  val equal : 'a t -> 'a t -> bool
  val compare: 'a t -> 'a t -> int
  val hash: 'a t -> int

  external get: 'a t -> ('a elem list * 'a elem list) list = "%identity"

  val empty : 'a t
  val full  : 'a t
  val cup   : 'a t -> 'a t -> 'a t
  val cap   : 'a t -> 'a t -> 'a t
  val diff  : 'a t -> 'a t -> 'a t
  val atom  : 'a elem -> 'a t

  val map : ('a elem-> 'b elem) -> 'a t -> 'b t
  val iter: ('a elem-> unit) -> 'a t -> unit
  val compute: empty:'d -> full:'c -> cup:('d -> 'c -> 'd) 
    -> cap:('c -> 'b -> 'c) -> diff:('c -> 'b -> 'c) ->
    atom:('a elem -> 'b) -> 'a t -> 'd
  val compute_bool: ('a elem -> 'b t) -> 'a t -> 'b t
    
  val print: string -> (Format.formatter -> 'a elem -> unit) -> 'a t ->
    (Format.formatter -> unit) list
    
    
  val trivially_disjoint : 'a t -> 'a t -> bool    
  val check: 'a t -> unit
end

module Make(X : SortedList.ARG) : S with type 'a elem = 'a X.t

include S with type 'a elem = 'a and type 'a t = ('a list * 'a list) list
