exception Error of string
open Ident

(* Pattern algebra *)

type descr
type node

val make: fv -> node
val define: node -> descr -> unit

val constr : Types.descr -> descr
val cup    : descr -> descr -> descr
val cap    : descr -> descr -> descr

val times  : node -> node -> descr 
val xml    : node -> node -> descr
val record : label -> node -> descr

val capture : id -> descr
val constant: id -> Types.const -> descr

val id: node -> int
val descr: node -> descr
val fv : node -> fv

(* Pattern matching: static semantics *)

val accept : node -> Types.node
val filter : Types.descr -> node -> (id * Types.node) list


(* Pattern matching: compilation *)

module Compile: sig
  type dispatcher

  type actions =
    | AIgnore of result
    | AKind of actions_kind
  and actions_kind = {
    basic: (Types.descr * result) list;
    atoms: result Atoms.map;
    chars: result Chars.map;
    prod: result dispatch dispatch;
    xml: result dispatch dispatch;
    record: record option;
  }
  and record =
    | RecLabel of label * result dispatch dispatch
    | RecNolabel of result option * result option
  and 'a dispatch =
    | Dispatch of dispatcher * 'a array
    | TailCall of dispatcher
    | Ignore of 'a
    | Impossible

  and result = int * source array
  and source = 
    | Catch | Const of Types.const 
    | Left of int | Right of int | Recompose of int * int

  val actions: dispatcher -> actions

  type 'a rhs = Match of int id_map * 'a | Fail
  val make_branches :
    Types.descr -> (node * 'a) list ->
    dispatcher * 'a rhs array

  val print_dispatcher: Format.formatter -> dispatcher -> unit

  val debug_compile : Format.formatter -> Types.node -> node list -> unit
end
