(**************************************************************************)
(*  The CDuce interpreter                                                 *)
(*  Alain.Frisch@ens.fr                                                   *)
(*  Copyright 2003 (see LICENSE for details)                              *)
(**************************************************************************)

module type ARG =
sig
    type 'a t
    val equal: 'a t -> 'a t -> bool
    val hash: 'a t -> int
    val compare: 'a t -> 'a t -> int
end

module type ARG0 =
sig
    type t
    val equal: t -> t -> bool
    val hash: t -> int
    val compare: t -> t -> int
end

module type S =
sig
  type 'a elem
  type 'a t
  val equal: 'a t -> 'a t -> bool
  val hash: 'a t -> int
  val compare: 'a t -> 'a t -> int

  external get: 'a t -> 'a elem list = "%identity"

  val singleton: 'a elem -> 'a t
  val iter: ('a elem -> unit) -> 'a t -> unit
  val filter: ('a elem -> bool) -> 'a t -> 'a t
  val exists: ('a elem -> bool) -> 'a t -> bool
  val fold: ('b -> 'a elem -> 'b) -> 'b -> 'a t -> 'b
  val pick: 'a t -> 'a elem option
  val length: 'a t -> int

  val empty: 'a t
  val is_empty: 'a t -> bool
  val from_list : 'a elem list -> 'a t
  val add: 'a elem -> 'a t -> 'a t
  val remove: 'a elem -> 'a t -> 'a t
  val disjoint: 'a t -> 'a t -> bool
  val cup: 'a t -> 'a t -> 'a t
  val split: 'a t -> 'a t -> 'a t * 'a t * 'a t
    (* split l1 l2 = (l1 \ l2, l1 & l2, l2 \ l1) *)
  val cap:  'a t -> 'a t -> 'a t
  val diff: 'a t -> 'a t -> 'a t
  val subset: 'a t -> 'a t -> bool
  val map: ('a elem-> 'b elem) -> 'a t -> 'b t
  val mem: 'a t -> 'a elem -> bool

  val check: 'a elem list -> unit

  module Map: sig
    type ('a,'b) map
    external get: ('a,'b) map -> ('a elem * 'b) list = "%identity"
    val empty: ('a,'b) map
    val iter: ('b -> unit) -> ('a,'b) map -> unit
    val filter: ('a elem -> 'b -> bool) -> ('a,'b) map -> ('a,'b) map
    val is_empty: ('a,'b) map -> bool
    val singleton: 'a elem -> 'b -> ('a,'b) map
    val assoc_remove: 'a elem -> ('a,'b) map -> 'b * ('a,'b) map
    val remove:  'a elem -> ('a,'b) map -> ('a,'b) map
    val merge: ('b -> 'b -> 'b ) -> ('a,'b) map -> ('a,'b) map -> ('a,'b) map
    val cap: ('b -> 'b -> 'b ) -> ('a,'b) map -> ('a,'b) map -> ('a,'b) map
    val sub: ('b -> 'b -> 'b ) -> ('a,'b) map -> ('a,'b) map -> ('a,'b) map

    val merge_elem: 'b -> ('a,'b) map -> ('a,'b) map -> ('a,'b) map
    val union_disj: ('a,'b) map -> ('a,'b) map -> ('a,'b) map
    val diff: ('a,'b) map -> 'a t -> ('a,'b) map
    val from_list: ('b -> 'b -> 'b ) -> ('a elem * 'b) list -> ('a,'b) map
    val from_list_disj: ('a elem * 'b) list -> ('a,'b) map

    val map_from_slist: ('a elem -> 'b) -> 'a t -> ('a,'b) map
    val collide: ('b -> 'c -> unit) -> ('a,'b) map -> ('a,'c) map -> unit
    val map: ('b -> 'c) -> ('a,'b) map -> ('a,'c) map
    val mapi: ('a elem -> 'b -> 'c) -> ('a,'b) map -> ('a,'c) map
    val constant: 'b -> 'a t -> ('a,'b) map
    val num: int -> 'a t -> ('a,int) map
    val map_to_list: ('b -> 'c) -> ('a,'b) map -> 'c list
    val mapi_to_list: ('a elem -> 'b -> 'c) -> ('a,'b) map -> 'c list
    val assoc: 'a elem -> ('a,'b) map -> 'b
    val assoc_present:  'a elem -> ('a,'b) map -> 'b
    val compare: ('b -> 'b -> int) -> ('a,'b) map -> ('a,'b) map -> int
    val hash: ('b -> int) -> ('a,'b) map -> int
    val equal: ('b -> 'b -> bool) -> ('a,'b) map -> ('a,'b) map -> bool
  end
end

module Make(X : ARG) : S with type 'a elem = 'a X.t
module Make_transp(X : ARG) : S with type 'a elem = 'a X.t and type 'a t = 'a X.t list
module Lift(X : ARG0) : ARG with type 'a t = X.t

module String : ARG0 with type t = string

